<?php

namespace Drupal\resourcespace_integration\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\image\Entity\ImageStyle;
use Drupal\media\Entity\Media;
use Symfony\Component\HttpFoundation\Request;
use GuzzleHttp\Exception\RequestException;
use Drupal\Core\Database\Database;
use Drupal\Core\Form\FormBuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ResourceSpaceFormController extends ControllerBase
{

    /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Constructs the controller.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   */
  public function __construct(FormBuilderInterface $form_builder) {
    $this->formBuilder = $form_builder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder')
    );
  }

  /**
   * Displays the ResourceSpace table with filters.
   */
  public function content() {
    // Render the filter form.
    $form = $this->formBuilder->getForm('Drupal\resourcespace_integration\Form\ResourceSpaceFilterForm');

   // Page size for pagination
    $page = \Drupal::request()->query->get('page', 0); // Get the current page from query params
    $limit = 10; // Number of rows to fetch per page
    $offset = $page * $limit; // Calculate the offset

    $filters = \Drupal::service('tempstore.private')->get('resourcespace_integration')->get('filters');
    $search ="";
    if (is_array($filters)) {
        $search = $filters['name'];
        
    }
    $resources = $this->getResources($offset, $limit, $search);
    $total = $resources['total'] ?? 0;
    $results = [];
    if ($resources['data'] && is_array($resources['data'])) {
        foreach ($resources['data'] as $resource) {
            $last_modified = $resource['file_modified'];
            $resource_date = $this->get_resource_date_by_ref_id($resource['ref']);
            $is_modified = FALSE;

            if ($resource_date) {
               $is_modified = $this->is_date_greater($last_modified, $resource_date);
            }
            $imageUrl= $this->getResoucePath($resource['ref'],$resource['file_extension']);
            $results[] = [
                'id' => $resource['ref'],
                'name' => $resource['field8'],
                'url' => $imageUrl,
                'is_modified' => $is_modified,
                'extension' => $resource['file_extension'],
                'modified_at' => $resource['file_modified'],
            ];
        }
    }
    

    // Calculate the number of pages
    $total_pages = ceil($total / $limit);

    // Generate pagination links (assuming you want to display simple next/previous links)
    if ($page > 0) {
    $previous_page_url = Url::fromRoute('<current>', ['page' => $page - 1])->toString();
    }

    if ($page < $total_pages - 1) {
    $next_page_url = Url::fromRoute('<current>', ['page' => $page + 1])->toString();
    }

  // Return the render array.
  return [
    '#theme' => 'resourcespace_content',
    '#form' => $form,
    '#data' => $results,
    '#next_page' => $next_page_url,
    '#previous_page' => $previous_page_url,
    '#attached' => [
        'library' => [
          'resourcespace_integration/resourcespace_integration_styles',
          'resourcespace_integration/resourcespace_integration.js'
        ],
      ],
  ];
  }

  /**
 * Compare two date strings and determine if the first date is greater than the second.
 *
 * @param string $date1
 *   The first date string to compare.
 * @param string $date2
 *   The second date string to compare.
 *
 * @return bool|null
 *   - TRUE if $date1 is greater than $date2.
 *   - FALSE if $date1 is less than or equal to $date2.
 *   - NULL if any of the dates are invalid.
 */
function is_date_greater($date1, $date2) {
    try {
      // Convert strings to DateTime objects.
      $datetime1 = new \DateTime($date1);
      $datetime2 = new \DateTime($date2);
  
      // Compare the dates and return the result.
      return $datetime1 > $datetime2;
    }
    catch (\Exception $e) {
      // Log an error if the date conversion fails.
      \Drupal::logger('resourcespace_integration')->error('Invalid date format: @date1 or @date2. Error: @error', [
        '@date1' => $date1,
        '@date2' => $date2,
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
 * Fetch the resource date by reference ID.
 *
 * @param string $ref_id
 *   The reference ID to search for.
 *
 * @return string|null
 *   The resource date, or NULL if not found.
 */
function get_resource_date_by_ref_id($ref_id) {
    // Load the entity query service.
    $entity_query = \Drupal::entityTypeManager()->getStorage('media')->getQuery();
  
    // Build the query.
    $entity_query->condition('field_ref_id', $ref_id)
    ->accessCheck(TRUE) ;
    
    $entity_query->range(0, 1); // Limit to one result for efficiency.
    $media_ids = $entity_query->execute();
  
    // If no media found, return NULL.
    if (empty($media_ids)) {
      return NULL;
    }
  
    // Load the media entity.
    $media_id = reset($media_ids);
    $media = \Drupal::entityTypeManager()->getStorage('media')->load($media_id);
  
    // Check if the field_resource_date exists and has a value.
    if ($media->hasField('field_resource_date') && !$media->get('field_resource_date')->isEmpty()) {
      return $media->get('field_resource_date')->value;
    }
  
    // Return NULL if no date is found.
    return NULL;
  }
  
     /**
     * Connect to ResourceSpace API, authenticate, and list resources for cron.
     */
    public function getResources($offset,$limit,$search="")
    {
        // Load configuration.
        $config = \Drupal::config('resourcespace_integration.settings');
        $apiUrl = $config->get('resourcespace_url');
        $username = $config->get('resourcespace_username');
        $apiKey = $config->get('resourcespace_api_key');

        if (!$apiUrl || !$username || !$apiKey) {
            return [];
        }

        $searchTerm = $search;

        $query = "user={$username}&function=do_search&fetchrows={$offset},{$limit}&search=" . urlencode($searchTerm);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";

        \Drupal::logger('resourcespace_integration')->error('Full url : ' . $fullUrl, ['@response' => print_r($fullUrl, true)]);

        // Make the API request.
        $response = $this->makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);

        if ($resources && is_array($resources)) {
            // foreach ($resources as $resource) {
            //     $this->createMediaEntity($resource);
            // }
        }
        return $resources;
    }

    public function getAllImageSizes($ref){
        // Load configuration.
        $config = \Drupal::config('resourcespace_integration.settings');
        $apiUrl = $config->get('resourcespace_url');
        $username = $config->get('resourcespace_username');
        $apiKey = $config->get('resourcespace_api_key');

        if (!$apiUrl || !$username || !$apiKey) {
            return [];
        }


        $query = "user={$username}&function=get_resource_all_image_sizes&ref=" . urlencode($ref_id);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";

        \Drupal::logger('resourcespace_integration')->error('Full url fo all sizes: ' . $fullUrl, ['@response' => print_r($fullUrl, true)]);

        // Make the API request.
        $response = $this->makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);
        \Drupal::logger('resourcespace_integration')->error('AI response : @response', ['@response' => print_r($resources, true)]);
        return $resources;
    }
    public function getResoucePath($ref_id,$extension){
        
        // Load configuration.
        $config = \Drupal::config('resourcespace_integration.settings');
        $apiUrl = $config->get('resourcespace_url');
        $username = $config->get('resourcespace_username');
        $apiKey = $config->get('resourcespace_api_key');

        if (!$apiUrl || !$username || !$apiKey) {
            return [];
        }


        $query = "user={$username}&function=get_resource_path&external=true&extension={$extension}&ref=" . urlencode($ref_id);

        // Generate the signature.
        $signature = hash('sha256', $apiKey . $query);

        // Full API URL with the signature.
        $fullUrl = "{$apiUrl}?{$query}&sign={$signature}";

        \Drupal::logger('resourcespace_integration')->error('Resource path url : ' . $fullUrl, ['@response' => print_r($fullUrl, true)]);

        // Make the API request.
        $response = $this->makeApiRequest($fullUrl);

        // Decode the JSON response.
        $resources = json_decode($response, true);
        \Drupal::logger('resourcespace_integration')->error('Rp response : @response', ['@response' => print_r($resources, true)]);

        return $resources;
    }

    /**
     * Authenticate with ResourceSpace and obtain a session API key.
     */
    private function authenticate($private_key, $username, $query)
    {

        $signature = hash('sha256', $private_key . $query);

        // Log the error response for debugging
       // \Drupal::logger('resourcespace_integration')->error('Signature : ' . $signature, ['@response' => print_r($signature, true)]);

        return $signature;
    }


    /**
     * Creates media entities for each image.
     */
    public function createMediaEntity($image_data)
    {

        // Log the incoming image data for debugging.
        $image_url = $image_data['url_scr'];
        $ref_id = $image_data['ref'];
        $modified_at = $image_data['file_modified'];
        if ($image_url && $ref_id) {
               
            try {
                // Check if the media entity with the same ref_id already exists.
                $query = \Drupal::entityQuery('media')
                    ->condition('bundle', 'image')
                    ->condition('field_ref_id', $ref_id)
                    ->accessCheck(false); 
                $existing_media_ids = $query->execute();

                if (!empty($existing_media_ids)) {
                    return;
                }
                $file = $this->save_image_to_media_library($image_url, $image_data['field8'] . ".jpg");
                // Step 2: Generate the thumbnail using an image style.
                $image_style = ImageStyle::load('thumbnail');

                $image_uri = $file->getFileUri();
                $destination_uri = $image_style->buildUri($file->uri->value);
                $image_style->createDerivative($image_uri, $destination_uri);

                // Create the media entity.
                \Drupal::logger('resourcespace_integration')->info($image_url);
                $media = \Drupal\media\Entity\Media::create([
                    'bundle' => 'image',
                    'name' => $file->getFilename(),
                    'field_ref_id' => $ref_id,
                    'field_modified_at' => $modified_at,
                    'field_remote_image_url' => $image_url,
                    'field_media_image' => [
                        'target_id' => $file->id(),
                        'alt' => 'Downloaded image',
                    ],
                ]);
                $media->save();

                // \Drupal::logger('resourcespace_integration')->info('Media entity created with ID @id.', ['@id' => $media->id()]);
            } catch (\Exception $e) {
                \Drupal::logger('resourcespace_integration')->error('Error creating media entity: @message', ['@message' => $e->getMessage()]);
            }
        } else {
            \Drupal::logger('resourcespace_integration')->error('Invalid image URL or missing ref_id.');
        }
    }

    public function save_image_to_media_library($image_url, $filename)
    {
        // Define the target directory in the public file system.
        $destination = 'public://images/';
        \Drupal::service('file_system')->prepareDirectory($destination, FileSystemInterface::CREATE_DIRECTORY);

        // Get the image data.
        $image_data = file_get_contents($image_url);
        if ($image_data === false) {
            \Drupal::logger('resourcespace_integration')->error('Failed to download image from @url', ['@url' => $image_url]);
            return null;
        }

        // Generate a unique file name.
        $file_uri = $destination . $filename;

        // Save the file.
        $file_system = \Drupal::service('file_system');
        $file_path = $file_system->saveData($image_data, $file_uri, FileSystemInterface::EXISTS_REPLACE);

        if (!$file_path) {
            \Drupal::logger('resourcespace_integration')->error('Failed to save the image file.');
            return null;
        }

        // Create a File entity.
        $file = File::create([
            'uri' => $file_path,
            'status' => 1,
        ]);
        $file->save();

        return $file;
    }


    private function makeApiRequest($url)
    {
        // Get the HTTP client service.
        $httpClient = \Drupal::httpClient();

        try {
            // Make the GET request.
            $response = $httpClient->get($url, [
                'verify' => false,
                'timeout' => 40,
            ]);

            // Return body as a string.
            return $response->getBody()->getContents();
        } catch (RequestException $e) {
            // Log the exception.
            \Drupal::logger('resourcespace_integration')->error('API request failed: @message', ['@message' => $e->getMessage()]);
            return null;
        }
    }

    /**
     * Handles AJAX requests.
     */
    public function ajaxCallback() {
        $data = \Drupal::request()->request->get('data');
        $rp= $this->getResoucePath(1001);
        return new JsonResponse([
        'message' => $this->t('AJAX request was successful! You sent: @data', ['@data' => $data]),
        ]);

    }

}